# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
__revision__ = '$Id: Agenda.py,v 1.1 2001/09/20 17:24:36 syt Exp $'

import time, string, calendar, os.path
from narval.lib import *
DATAFILE = os.path.join(config.get_home(),'data/agenda.xml')
DATE_FORMAT = '%Y/%m/%d %H:%M'    
DATE_TODAY = time.localtime(time.time())

#  Agenda Interface:
#     def add(self,node): pass
#     # return nothing
#     def remove(self,node): pass
#     # return nothing
#     def modify(self,node_after,node_before): pass
#     # return nothing
#     def update(self,list_of_extern_ad_books,cgi_request): pass
#     # returns nothing
#     def get_events(self,doc,day): pass
#     # return a list of events nodes contained at the date day


def next_day(date) :
    day = date[2] + 1
    if day > calendar.monthrange(date[0],date[1])[1] :
        day = 1
    return (date[0], date[1], day, date[3], date[4],
                        date[5], date[6], date[7], date[8]) 
    
def prev_day(date) :
    day = date[2] - 1
    if day < 1 :
        day = calendar.monthrange(date[0],date[1])[1]
    return (date[0], date[1], day, date[3], date[4],
                        date[5], date[6], date[7], date[8]) 
    
def next_month(date) :
    next_year = date[0]
    next_month = date[1] + 1
    if next_month > 12 :
        next_month = 1
        next_year = next_year + 1
    return (next_year, next_month, 1, date[3], date[4],
            date[5], date[6], date[7], date[8])

    
def prev_month(date) :
    next_year = date[0]
    next_month = date[1] - 1
    if next_month < 1 :
        next_month = 12
        next_year = next_year - 1
    return (next_year, next_month, 1, date[3], date[4],
            date[5], date[6], date[7], date[8])

    
class Agenda:

    def __init__(self) :
        try:
            self.document = read_domtree_from_file(DATAFILE)
        except IOError :
            file = open(DATAFILE,'w')
            file.write('<?xml version="1.0" ?><agenda></agenda>')
            file.close()
            self.document = read_domtree_from_file(DATAFILE)
        self.tree = self.document.documentElement
        self.response = self.month_html()
      


    def write_file(self) :
        for e in select_many(self.tree,'entry') :
            if not e.hasChildNodes:
                self.tree.removeChild(e)
        write_domtree_to_file(self.tree, DATAFILE)
       

    def process_input(self,request) :
        """Guess what... this processes the input!"""
        doc = request.ownerDocument
        list_op = select_many(request,'cgi-item[@key="op"]/text()')
        # request was empty, quit
        if not list_op:
            return
        for op in list_op:
            # operation is in {add,remove,goto}
            if op.data == 'ADD' :
                date_str = "%s.%s.%s.%s" % \
                           (select_one(request,'cgi-item[@key="year"]/text()').data,
                            select_one(request,'cgi-item[@key="month"]/text()').data,
                            select_one(request,'cgi-item[@key="day"]/text()').data,
                            select_one(request,'cgi-item[@key="hour"]/text()').data)
                date_tuple = time.strptime(date_str,'%Y.%B.%d.%H:%M')
                date_given = time.strftime(DATE_FORMAT,date_tuple)
                
                # FIXME:partie  amliorer. verif que jour correspond au No,
                # mettre valeur par dfaut pour l'anne...
                entry = doc.createElementNS('','entry')
                entry.setAttributeNS('','date',date_given)
                action =select_one(request,'cgi-item[@key="action"]/text()').data 
                entry.appendChild(doc.createTextNode(action))
                entry = self.tree.ownerDocument.importNode(entry,DEEP_COPY)
                self.tree.appendChild(entry)
                # write modified agenda to file
                self.write_file()
                self.response = self.month_html(date_tuple)
                
            elif op.data == 'GOTO':
                #Selon ce que l'utilisateur aura entr,
                #goto affichera la page du jour ou du mois
                try:
                    date_str = "%s.%s.%s" % \
                               (select_one(request,'cgi-item[@key="year"]/text()').data,
                                select_one(request,'cgi-item[@key="month"]/text()').data,
                                select_one(request,'cgi-item[@key="day"]/text()').data)
                    date_tuple = time.strptime(date_str,'%Y.%B.%d')
                    # write modified agenda to file
                    self.write_file()
                    self.response=self.day_html(date_tuple)
                except SelectException:
                    date_str = "%s.%s"%(select_one(request,'cgi-item[@key="year"]/text()').data,
                                        select_one(request,'cgi-item[@key="month"]/text()').data)
                    date_tuple = time.strptime(date_str,'%Y.%B')
                    # write modified agenda to file
                    self.write_file()
                    self.response = self.month_html(date_tuple)
                 

            elif op.data == 'REMOVE' :
                date = ''
                for ui in select_many(request,'cgi-item[@key="check"]/text()'):
                    d = ui.data
                    for c in select_many(self.tree,'entry[@date="%s"]'%(d)):
                        if not c.hasChildNodes :
                            self.tree.removeChild(c)
                        else:
                            self.tree.removeChild(c)
                    date = time.strptime(d,DATE_FORMAT)
                # write modified agenda to file
                self.write_file()
                if date: self.response = self.day_html(date)
                else: return
                
        
                
           
            else :
                date = op.data
                try:
                    new_date = time.strptime (date,'%d %B %Y')
                    # write modified agenda to file
                    self.write_file()
                    self.response = self.day_html(new_date) 
                except ValueError:
                    new_date = time.strptime (date,'%B %Y')
                    # write modified agenda to file
                    self.write_file()
                    self.response = self.month_html(new_date)       

            # J'ai mis cette ligne dans chaque partie plutot qu' la fin,
            #Car il faut que l'arbre soit modifi avant que l'on appelle day/month_html
            # write modified agenda to file
            #self.write_file()


    def add(self,node):
        entry = self.tree.ownerDocument.importNode(node,DEEP_COPY)
        self.tree.appendChild(entry)

    def remove(self,node):
        self.tree.removeChild(node)
    
    def modify(self,node_after,node_before): pass
    # return nothing
    def update(self,list_of_extern_ad_books,cgi_request): pass
    # returns nothing
    def get_events(self,doc,day): pass
    # return a list of events nodes contained at the date day


    def month_html(self, date=DATE_TODAY) :
        """Ici date est un TUPLE!!!
        cette fonction gnre dynamiquement une page html prsentant sous forme
        de tableau les taches  effectuer au cours d'un mois donn en entre.
        Cette page est gnre  partir de la page accueil.html qui apparat
        automatiquement lorsque Narval fait appel  l'Agenda
        """
        import urllib

        first_day_num, month_length = calendar.monthrange(date[0],date[1])

        # dbut du document html.
        # on cre un tableau de 7 colonnes, chacune ayant de nom d'un jour
        temp = \
             '''
             <form action=agenda method=get>
             <h2>Planning for %s</h2><br>
             <center>
             <table border=3 cellspacing=2 cellpading=5>
             <tr>
             <th>%s</th>
             <th>%s</th>
             <th>%s</th>
             <th>%s</th>
             <th>%s</th>
             <th>%s</th>
             <th>%s</th>
             </tr><tr>\n''' % (time.strftime('%B %Y',date),\
                               time.strftime('%A',(0,0,0,0,0,0,6,0,0)),\
                               time.strftime('%A',(0,0,0,0,0,0,0,0,0)),\
                               time.strftime('%A',(0,0,0,0,0,0,1,0,0)),\
                               time.strftime('%A',(0,0,0,0,0,0,2,0,0)),\
                               time.strftime('%A',(0,0,0,0,0,0,3,0,0)),\
                               time.strftime('%A',(0,0,0,0,0,0,4,0,0)),\
                               time.strftime('%A',(0,0,0,0,0,0,5,0,0)))
        
        # le premier du mois n'est pas obligatoirement un dimanche
        # --> cre autant de cases vides que ncessaire
        # rmq: monday = 0, sunday = 6 
        temp = temp + '<td></td>\n'*(first_day_num+1)
           
        # affiche une case par jour du mois
        for day_num in range(1,month_length+1) :
            # si on arrive  la fin de la semaine, on passe  la ligne
            if (day_num+first_day_num)%7 == 0 :
                temp = temp + '\n</tr>\n<tr>\n'

            # on construit la liste trie des vnements
            entry_list = select_many(self.tree,
                                     'entry[starts-with(string(@date),"%i/%02i/%02i")]'\
                                          %(date[0],date[1],day_num))
            # si un vnement est prvu ce jour, on colore la case
            if entry_list :
                temp = temp + \
                       '''<td bgcolor="#FFFF99" align=center>
                       <a href="agenda?op=GOTO&year=%i&month=%s&day=%i">%i</a>
                       </td>\n
                       ''' % (date[0],
                              time.strftime('%B',date),day_num,day_num)
            else :
                temp = temp + '<td align=center>%i</td>\n' % (day_num)
           
        # creation des boutons permettant d'aller au mois prcdent et
        # au mois suivant, ainsi que d'un bouton permettant de rentrer de
        # nouvelles donnes
        temp = temp + \
               '''</tr>\n</table>
               <form action="agenda" method="get">
               <input type="submit" name="op" value="%s">
               <input type="submit" name="op" value="%s">
               ''' % (time.strftime('%B %Y',prev_month(date)),
                      time.strftime('%B %Y',next_month(date)))

        temp = temp + '<hr>'
        temp = temp + self.form_common(date)
        temp = temp +'</form></center>\n</body></html>'

        return temp


    def form_common(self,date) :
        temp = '''<input type="text" size=5 value="%s" name="year">
               / <select name="month">
               ''' % (time.strftime('%Y',date))
        
        monthes = []
        for i in range(1,13) :
            monthes.append(time.strftime('%B',(0,i,0,0,0,0,0,0,0)))
        
        for month in monthes:
            temp = temp + '<option'
            if month == time.strftime('%B',date):
                temp = temp + ' selected'
            temp = temp + '>' +month+ '</option>\n'

        temp = temp + '</select>\n'
        temp = temp + \
               '''/ <input type="text" size=5 value="%i" name="day">
               / <input type="text" size=5 value="%s" name="hour">
               <input size="10" type="submit" name="op" value="GOTO">
               <br>
               <font color="#008c95"><b>Event</b></font>
               <input type="text" size="100" name="action">
               <br>
               <input type="submit" name="op" value="ADD">
               ''' % (date[2], time.strftime('%H:%M',date))

        return temp
    
    
    def day_html(self,date=DATE_TODAY) :
        """Cette fonction gnre dynamiquement une page html permettant
        d'afficher le planning d'un jour donn"""
                
        # dbut de la source html
        temp = \
             '''
             <h2>Planning for %s</h2>
             <br>
             <form action="agenda" method="GET">\n
             ''' % (time.strftime ('%A the %d of %B %Y',date))

        # On parcourt la liste des vnements trie par ordre chronologique
        # on garde les vnements ayant comme attribut date la date du jour
        # affich.On les affiche. Si aucun vnement, Narval renvoie
        # NOTHING TO DO

        # on construit la liste trie des vnements
        entry_list = select_many(self.tree,'entry[starts-with(string(@date),"%s")]'%\
                                 (time.strftime('%Y/%m/%d',date)))
        entry_list.sort()
        if entry_list :
            temp = temp + '<ul>\n'
            for entry in entry_list :
                temp = temp + \
                       '''<li><input type="checkbox" value="%s" name="check">\
                       at %s: %s</li>\n
                       ''' %  (entry.getAttributeNS('','date'),
                               entry.getAttributeNS('','date')[-5:],
                               entry.firstChild.data)
            temp = temp + '</ul>'
        else:
            temp  = temp + '<td>NOTHING TO DO TODAY</td>\n'
            

        # FIXME: il faudrait que les vnement se mettent dans la ligne
        # approprie selon l'heure  laquelle ils sont programms.
        #         temp = temp + \
        #                '''<center>
        #                <table border="3" cellspacing="2" cellpading=5>\n
        #                <tr><th>Earlier</th></tr>\n
        #                <tr><th>Between 8 and 10</th></tr>\n
        #                <tr><th>Between 10 and 12</th></tr>\n
        #                <tr><th>Between 12 and 14</th></tr>\n
        #                <tr><th>Between 14 and 16</th></tr>\n
        #                <tr><th>Between 16 and 18</th></tr>\n
        #                <tr><th>Between 18 and 20</th></tr>\n
        #                <tr><th>Between 20 and 22</th></tr>\n
        #                <tr><th>Later</th></tr>\n
        #                </table>\n'''
       

        # formulaire permettant d'aller aux jours suivants, prcdents,\
        # d'ajouter un vnement...

        temp = temp + \
               '''<center>
               <a href="agenda?month=%s&year=%s&op=GOTO">See the whole month?</a><br>\n
               <input type="submit" name="op" value="REMOVE"><hr>
               <form action="agenda" method="GET">\n
               ''' % (time.strftime('%B',date),
                      time.strftime('%Y',date))

        temp =  temp + \
               '''<input type="submit" name="op" value="%s">\n
               <input type="submit" name="op" value="%s"><br>\n
               ''' % (time.strftime('%d %B %Y',prev_day(date)),
                      time.strftime('%d %B %Y',next_day(date)))

        temp = temp + self.form_common(date)
               
        temp = temp + "</form></center>\n</body></html>"
        return temp


    def tomorrow(self) :
        import time
        demain = time.localtime(time.time()+60*60*24)
        tomo = string.lower(time.strftime("%A", demain))
        rep = ''
        if self.events.has_key(tomo) :
            rep = rep + 'Tomorrow on '+tomo+' remember to\n'
            for e in self.events[tomo] :
                rep = rep + '\t'+e+'\n'
        return rep



## MODULE AUTOTEST
if __name__ == "__main__" :
    import sys
    sys.stderr.write("Narval Agenda module autotest\n---------------------\n")
    agd = Agenda()
    print agd.tomorrow()
